@@ -18,18 +18,43 @@
 ##' @param dir.sims The directory in which NMsim will store all
 ##'     generated files. Default is to create a folder called `NMsim`
 ##'     next to `file.mod`.
+##' @param dir.res Provide a path to a directory in which to save rds
+##'     files with paths to results. Default is to use dir.sims. After
+##'     running `NMreadSim()` on these files, the original simulation
+##'     files can be deleted. Hence, providing both `dir.sims` and
+##'     `dir.res` provides a structure that is simple to
+##'     clean. `dir.sims` can be purged when `NMreadSim` has been run
+##'     and only small `rds` and `fst` files will be kept in
+##'     `dir.res`. Notice, in case multiple models are simulated,
+##'     multiple `rds` (to be read with `NMreadSim()`) files will be
+##'     created by default. In cases where multiple models are
+##'     simulated, see `file.res` to get just one file refering to all
+##'     simulation results.
 ##' @param name.sim Give all filenames related to the simulation a
 ##'     suffix. A short string describing the sim is recommended like
 ##'     "ph3_regimens".
-##' @param order.columns reorder columns by calling
-##'     \code{NMdata::NMorderColumns} before saving dataset and
-##'     running simulations? Default is TRUE.
-##' @param script The path to the script where this is run. For
-##'     stamping of dataset so results can be traced back to code.
 ##' @param subproblems Number of subproblems to use as
 ##'     \code{SUBPROBLEMS} in \code{$SIMULATION} block in Nonmem. The
 ##'     default is subproblem=0 which means not to use
 ##'     \code{SUBPROBLEMS}.
+##' @param table.vars Variables to be printed in output table as a
+##'     character vector or a space-separated string of variable
+##'     names. The default is to export the same tables as listed in
+##'     the input control stream. If \code{table.vars} is provided,
+##'     all output tables in estimation control streams are dropped
+##'     and replaced by a new one with just the provided variables. If
+##'     many variables are exported, and much fewer are used, it can
+##'     speed up NMsim significantly to only export what is needed
+##'     (sometimes this is as little as "PRED IPRED"). Nonmem writes
+##'     data slowly so reducing output data can make a very big
+##'     difference in execution time. See \code{table.options} too.
+##' @param table.options A character vector or a string of
+##'     space-separated options. Only used if \code{table.vars} is
+##'     provided. If constructing a new output table with
+##'     \code{table.vars} the default is to add two options,
+##'     \code{NOAPPEND} and \code{NOPRINT}. You can modeify that with
+##'     \code{table.options}. Do not try to modify output filename -
+##'     \code{NMsim} takes care of that.
 ##' @param reuse.results If simulation results found on file, should
 ##'     they be used? If TRUE and reading the results fail, the
 ##'     simulations will still be rerun.
@@ -62,33 +87,19 @@
 ##'
 ##' In case \code{method.sim=NMsim_EBE}, seeds are not used.
 ##'
-##' @param seed Deprecated. See \code{seed.R} and \code{seed.nm}.
+##' @param order.columns reorder columns by calling
+##'     \code{NMdata::NMorderColumns} before saving dataset and
+##'     running simulations? Default is TRUE.
+##' @param script The path to the script where this is run. For
+##'     stamping of dataset so results can be traced back to code.
 ##' @param args.psn.execute A charachter string that will be passed as
 ##'     arguments PSN's `execute`.
-##' @param table.vars Variables to be printed in output table as a
-##'     character vector or a space-separated string of variable
-##'     names. The default is to export the same tables as listed in
-##'     the input control stream. If \code{table.vars} is provided,
-##'     all output tables in estimation control streams are dropped
-##'     and replaced by a new one with just the provided variables. If
-##'     many variables are exported, and much fewer are used, it can
-##'     speed up NMsim significantly to only export what is needed
-##'     (sometimes this is as little as "PRED IPRED"). Nonmem writes
-##'     data slowly so reducing output data can make a big difference
-##'     in execution time. See \code{table.options} too.
-##' @param table.options A character vector or a string of
-##'     space-separated options. Only used if \code{table.vars} is
-##'     provided. If constructing a new output table with
-##'     \code{table.vars} the default is to add two options,
-##'     \code{NOAPPEND} and \code{NOPRINT}. You can modeify that with
-##'     \code{table.options}. Do not try to modify output filename -
-##'     \code{NMsim} takes care of that.
-##' @param text.table A character string including the variables to
-##'     export from Nonmem.
 ##' @param text.sim A character string to be pasted into
 ##'     $SIMULATION. This must not contain seed or SUBPROBLEM which
-##'     are handled separately. Default is to include "ONLYSIM". To
-##'     avoid that, use text.sim="".
+##'     are handled separately. Default is to include "ONLYSIM". You
+##'     cannot avoid that using `text.sim`. If you need to drop
+##'     `ONLYSIM`, use
+##'     `modify.model=list(simulation=list(overwrite("ONLYSIM","")))`.
 ##' @param method.sim A function (not quoted) that creates the
 ##'     simulation control stream and other necessary files for a
 ##'     simulation based on the estimation control stream, the data,
@@ -97,8 +108,9 @@
 ##'     step. See details section on oter methods, and see examples
 ##'     and especially vignettes on how to use the different provided
 ##'     methods.
-##' @param typical Run with all ETAs fixed to zero? Technically all ETAs=0 is obtained by replacing
-##' \code{$OMEGA} by a zero matrix. Default is FALSE. 
+##' @param typical Run with all ETAs fixed to zero? Technically all
+##'     ETAs=0 is obtained by replacing \code{$OMEGA} by a zero
+##'     matrix. Default is FALSE.
 ##' @param execute Execute the simulation or only prepare it?
 ##'     `execute=FALSE` can be useful if you want to do additional
 ##'     tweaks or simulate using other parameter estimates.
@@ -106,25 +118,46 @@
 ##'     useful if creating a large number of simulations,
 ##'     e.g. simulate with all parameter estimates from a bootstrap
 ##'     result.
-##' @param col.row Only used if data is not supplied (which is most
-##'     likely for simulations for VPCs) A column name to use for a
-##'     row identifier. If none is supplied,
-##'     \code{NMdataConf()[['col.row']]} will be used. If the column
-##'     already exists in the data set, it will be used as is, if not
-##'     it will be added.
+##' @param nc Number of cores used in parallelization. Only used if
+##'     `sge=TRUE`.
+##' @param wait Wait for simulations to finish? Default is to do so if
+##'     simulations are run locally but not to if they are sent to the
+##'     cluster. Waiting for them means that the results will be read
+##'     when simulations are done. If not waiting, path(s) to `rds`
+##'     files to read will be returned. Pass them through
+##'     `NMreadSim()` (which also supports waiting for the simulations
+##'     to finish).
 ##' @param method.execute Specify how to call Nonmem. Options are
 ##'     "psn" (PSN's execute), "nmsim" (an internal method similar to
 ##'     PSN's execute), and "direct" (just run Nonmem directly and
 ##'     dump all the temporary files). "nmsim" has advantages over
 ##'     "psn" that makes it the only supported method when
-##'     type.sim="NMsim_EBE". "psn" has the simple advantage that
-##'     the path to nonmem does not have to be specified if "execute"
-##'     is in the system search path. So as long as you know where
-##'     your Nonmem executable is, "nmsim" is recommended. The default
-##'     is "nmsim" if path.nonmem is specified, and "psn" if not.
-##' @param nc Number of cores used in parallelization. This is so far
-##'     only supported with \code{method.execute="psn"}.
-##' @param method.update.inits The initial values of all parameters
+##'     type.sim="NMsim_EBE". "psn" has the simple advantage that the
+##'     path to nonmem does not have to be specified if "execute" is
+##'     in the system search path. So as long as you know where your
+##'     Nonmem executable is, "nmsim" is recommended. The default is
+##'     "nmsim" if path.nonmem is specified, and "psn" if not.
+##' @param inits Control the parameter values. `inits` is a list. The
+##'     `method` element controls which method is used to do this, and
+##'     this corresponds to the old `method.update.inits` argument. If
+##'     using the new `method=nmsim2` you can specify parameter
+##'     values, fix/unfix them, and edit lower and upper limits for
+##'     estimation. If inits contains `method=nmsim2`, all other
+##'     arguments are passed to `NMwriteInits`. This is a flexible
+##'     method that allows for modification of the parameter values
+##'     and is expected to be the default method in the
+##'     future. Example which will update the parameter values based
+##'     on the available estimate, but with `THETA(2)=1.3`:
+##'     `inits=list(method="nmsim2","THETA(2)"=list(init=1.3))`. See
+##'     `?NMwriteInits` too.
+##'
+##' See `method.update.inits` methods for other methods available. See
+##' also `file.ext` which can now be handled by `inits` too. This
+##' change is done because it collects the update of the "initial"
+##' parameter values into one interface rather than multiple
+##' arguments.
+##' 
+##' @param method.update.inits This argument is being deprecated, please migrate to `inits` instead. The initial values of all parameters
 ##'     are by updated from the estimated model before running the
 ##'     simulation. NMsim can do this with a native function or use
 ##'     PSN to do it - or the step can be skipped to not update the
@@ -156,25 +189,29 @@
 ##' "nmsim" is the default.
 ##' 
 ##' }
-##' 
+##'
+##' @param path.nonmem The path to the Nonmem executable to use. The
+##'     could be something like "/usr/local/NONMEM/run/nmfe75" (which
+##'     is a made up example). No default is available. You should be
+##'     able to figure this out through how you normally execute
+##'     Nonmem, or ask a colleague.
 ##' @param dir.psn The directory in which to find PSN's executables
 ##'     ('execute' and 'update_inits'). The default is to rely on the
 ##'     system's search path. So if you can run 'execute' and
 ##'     'update_inits' by just typing that in a terminal, you don't
 ##'     need to specify this unless you want to explicitly use a
 ##'     specific installation of PSN on your system.
-##' @param path.nonmem The path to the Nonmem executable to use. The
-##'     could be something like "/usr/local/NONMEM/run/nmfe75" (which
-##'     is a made up example). No default is available. You should be
-##'     able to figure this out through how you normally execute
-##'     Nonmem, or ask a colleague.
 ##' @param modify.model Named list of additional control stream
 ##'     section edits. Note, these can be functions that define how to
 ##'     edit sections. This is an advanced feature which is not needed
 ##'     to run most simulations. It is however powerful for some types
 ##'     of analyses, like modifying parameter values. See vignettes
 ##'     for further information.
-##' @param list.sections Deprecated. Use modify.model instead.
+##' @param sizes If needed, adjust the `$SIZES` section by providing a
+##'     list of arguments to `NMupdateSizes()`. Example:
+##'     `sizes=list(PD=80)`. See `?NMupdateSizes` for details. Don't
+##'     use arguments like `file.mod` and `newfile` which are handled
+##'     internally.
 ##' @param create.dirs If the directories specified in dir.sims and
 ##'     dir.res do not exists, should it be created? Default is TRUE.
 ##' @param sim.dir.from.scratch If TRUE (default) this will wipe the
@@ -191,13 +228,13 @@
 ##'     1000 replications, fixed something and now rand 500. If you
 ##'     choose FALSE here, you can end up with the results of 500 new
 ##'     and 500 old simulations.
-##' @param file.ext Optionally provide a parameter estimate file from
-##'     Nonmem. This is normally not needed since `NMsim` will by
-##'     default use the ext file stored next to the input control
-##'     stream (replacing the file name extension with `.ext`). If
-##'     using method.update.inits="psn", this argument cannot be
-##'     used. If you want provide parameters to be used for the
-##'     simulation, look at the `ext` argument to `NMsim_VarCov`.
+##' @param file.ext Depecated. Use
+##'     `inits=list(file.ext="path/to/file.ext") instead`. Optionally
+##'     provide a parameter estimate file from Nonmem. This is
+##'     normally not needed since `NMsim` will by default use the ext
+##'     file stored next to the input control stream (replacing the
+##'     file name extension with `.ext`). If using
+##'     method.update.inits="psn", this argument cannot be used. 
 ##' @param auto.dv Add a column called `DV` to input data sets if a
 ##'     column of that name is not found? Nonmem is generally
 ##'     dependent on a `DV` column in input data but this is typically
@@ -206,36 +243,6 @@
 ##'     \code{auto.dv=TRUE} and no `DV` column is found, `DV=NA` will
 ##'     be added. In this case (`auto.dv=TRUE` and no `DV` column
 ##'     found) a `MDV=1` column will also be added if none found.
-##' @param as.fun The default is to return data as a data.frame. Pass
-##'     a function (say `tibble::as_tibble`) in as.fun to convert to
-##'     something else. If data.tables are wanted, use
-##'     as.fun="data.table". The default can be configured using
-##'     NMdataConf.
-##' @param args.NMscanData If \code{execute=TRUE&sge=FALSE}, NMsim
-##'     will normally read the results using \code{NMreadSim}. Use
-##'     this argument to pass additional arguments (in a list) to that
-##'     function if you want the results to be read in a specific
-##'     way. This can be if the model for some reason drops rows, and
-##'     you need to merge by a row identifier. You would do
-##'     `args.NMscanData=list(col.row="ROW")` to merge by a column
-##'     called `ROW`. This is only used in rare cases.
-##'
-##' @param system.type A charachter string, either \"windows\" or
-##'     \"linux\" - case insensitive. Windows is only experimentally
-##'     supported. Default is to use \code{Sys.info()[["sysname"]]}.
-##' @param suffix.sim Deprecated. Use name.sim instead.
-##' @param dir.res Provide a path to a directory in which to save rds
-##'     files with paths to results. Default is to use dir.sims. After
-##'     running `NMreadSim()` on these files, the original simulation
-##'     files can be deleted. Hence, providing both `dir.sims` and
-##'     `dir.res` provides a structure that is simple to
-##'     clean. `dir.sims` can be purged when `NMreadSim` has been run
-##'     and only small `rds` and `fst` files will be kept in
-##'     `dir.res`. Notice, in case multiple models are simulated,
-##'     multiple `rds` (to be read with `NMreadSim()`) files will be
-##'     created by default. In cases where multiple models are
-##'     simulated, see `file.res` to get just one file refering to all
-##'     simulation results.
 ##' @param file.res Path to an rds file that will contain a table of
 ##'     the simulated models and other metadata. This is needed for
 ##'     subsequently retrieving all the results using
@@ -247,13 +254,6 @@
 ##'     file containing information about all simulated models will be
 ##'     created. Notice if \code{file.res} is supplied, \code{dir.res}
 ##'     is not used.
-##' @param wait Wait for simulations to finish? Default is to do so if
-##'     simulations are run locally but not to if they are sent to the
-##'     cluster. Waiting for them means that the results will be read
-##'     when simulations are done. If not waiting, path(s) to `rds`
-##'     files to read will be returned. Pass them through
-##'     `NMreadSim()` (which also supports waiting for the simulations
-##'     to finish).
 ##' @param clean The degree of cleaning (file removal) to do after
 ##'     Nonmem execution. If `method.execute=="psn"`, this is passed
 ##'     to PSN's `execute`. If `method.execute=="nmsim"` a similar
@@ -275,6 +275,28 @@
 ##'     it is only recommended to disable this if you are fully aware
 ##'     of such a feature of your control stream, you know how it
 ##'     impacts simulation, and you want to get rid of warnings.
+##' @param col.row Only used if data is not supplied (which is most
+##'     likely for simulations for VPCs) A column name to use for a
+##'     row identifier. If none is supplied,
+##'     \code{NMdataConf()[['col.row']]} will be used. If the column
+##'     already exists in the data set, it will be used as is, if not
+##'     it will be added.
+##' @param as.fun The default is to return data as a data.frame. Pass
+##'     a function (say `tibble::as_tibble`) in as.fun to convert to
+##'     something else. If data.tables are wanted, use
+##'     as.fun="data.table". The default can be configured using
+##'     NMdataConf.
+##' @param args.NMscanData If \code{execute=TRUE&sge=FALSE}, NMsim
+##'     will normally read the results using \code{NMreadSim}. Use
+##'     this argument to pass additional arguments (in a list) to that
+##'     function if you want the results to be read in a specific
+##'     way. This can be if the model for some reason drops rows, and
+##'     you need to merge by a row identifier. You would do
+##'     `args.NMscanData=list(col.row="ROW")` to merge by a column
+##'     called `ROW`. This is only used in rare cases.
+##' @param system.type A charachter string, either \"windows\" or
+##'     \"linux\" - case insensitive. Windows is only experimentally
+##'     supported. Default is to use \code{Sys.info()[["sysname"]]}.
 ##' @param format.data.complete For development purposes - users do
 ##'     not need this argument. Controls what format the complete
 ##'     input data set is saved in.  Possible values are `rds`
@@ -283,6 +305,12 @@
 ##'     input data to output data. `csv` will also lead to loss of
 ##'     additional information such as factor levels.
 ##' @param ... Additional arguments passed to \code{method.sim}.
+##' @param list.sections Deprecated. Use modify.model instead.
+##' @param suffix.sim Deprecated. Use name.sim instead.
+##' @param text.table A character string including the variables to
+##'     export from Nonmem. Deprecated. Use `table.vars` and
+##'     `table.options` instead.
+##' @param seed Deprecated. See \code{seed.R} and \code{seed.nm}.
 ##' @return A data.frame with simulation results (same number of rows
 ##'     as input data). If `sge=TRUE` a character vector with paths to
 ##'     simulation control streams.
@@ -371,9 +399,11 @@ NMsim <- function(file.mod,data,dir.sims, name.sim,
                   execute=TRUE,sge=FALSE,
                   nc=1,transform=NULL,
                   method.execute,
+                  inits,
                   method.update.inits,
                   create.dirs=TRUE,dir.psn,
                   modify.model,
+                  sizes,
                   sim.dir.from.scratch=TRUE,
                   col.row,
                   args.NMscanData,
@@ -464,19 +494,6 @@ NMsim <- function(file.mod,data,dir.sims, name.sim,
     
     ## Section end: Dummy variables, only not to get NOTE's in pacakge checks
 
-    paste.begin <- function(x,add,...){
-        res <- paste(add,x[1],...)
-        if(length(x)>1){
-            res <- c(res,x[2:(length(x))])
-        }
-        res
-    }
-
-    paste.end <- function(x,add,...){
-        c(x[0:(length(x)-1)],
-          paste(x[length(x)],add,...)
-          )
-    }
 
     ## as.fun
     if(missing(as.fun)) as.fun <- NULL
@@ -512,7 +529,7 @@ NMsim <- function(file.mod,data,dir.sims, name.sim,
     if(missing(path.nonmem)) path.nonmem <- NULL
     if(missing(method.execute)) method.execute <- NULL
     ## NMsimConf <- NMsimTestConf(path.nonmem=path.nonmem,dir.psn=dir.psn,method.execute=method.execute,must.work=execute)
-NMsimConf <- NMsimTestConf(path.nonmem=path.nonmem,dir.psn=dir.psn,method.execute=method.execute,must.work=FALSE)
+    NMsimConf <- NMsimTestConf(path.nonmem=path.nonmem,dir.psn=dir.psn,method.execute=method.execute,must.work=FALSE)
 
     
     ## after definition of wait and wait.exec, wait is used by
@@ -527,19 +544,15 @@ NMsimConf <- NMsimTestConf(path.nonmem=path.nonmem,dir.psn=dir.psn,method.execut
 
     ## args.psn.execute
     if(missing(args.psn.execute)) args.psn.execute <- NULL
-    ## args.psn.execute <- simpleCharArg("args.psn.execute"
-    ##                                  ,args.psn.execute
-    ##                                  ,default="-clean=5 -model_dir_name -nm_output=xml,ext,cov,cor,coi,phi"
-    ##                                  ,accepted=NULL
-    ##                                  ,clean=FALSE
-    ##                                  ,lower=FALSE)
     if(missing(clean)) clean <- 1
     
-
     if(missing(file.ext)) file.ext <- NULL
-    
+
+    if(missing(inits)) inits <- NULL
     if(missing(method.update.inits)) method.update.inits <- NULL
-    method.update.inits <- adjust.method.update.inits(method.update.inits,system.type=NMsimConf$system.type,dir.psn=NMsimConf$dir.psn,cmd.update.inits=cmd.update.inits,file.ext=file.ext)
+    inits <- adjust.method.update.inits(method.update.inits,system.type=NMsimConf$system.type,dir.psn=NMsimConf$dir.psn,cmd.update.inits=cmd.update.inits,file.ext=file.ext,inits=inits)
+    method.update.inits <- inits$method
+    file.ext <- inits$file.ext
     
     
 
@@ -705,7 +718,7 @@ NMsimConf <- NMsimTestConf(path.nonmem=path.nonmem,dir.psn=dir.psn,method.execut
         tabv2 <- paste(table.vars,collapse=" ")
         tabv2 <- gsub(" +"," ",tabv2 )
         if(length(strsplit(tabv2," ")[[1]])<3){
-            message("Using less than three variables in table.vars in combination with subproblems may cause issues with NMdata versions <=0.1.6. If you get an error, try to add any variable or two to table.vars.")
+            message("Using less than three variables in table.vars in combination with subproblems may cause issues. If you get an error, try to add any variable or two to table.vars.")
         }
     }
     
@@ -760,6 +773,10 @@ NMsimConf <- NMsimTestConf(path.nonmem=path.nonmem,dir.psn=dir.psn,method.execut
 
     }
     if(!is.null(data) && is.list(data) && !is.data.frame(data)) {
+        ## if the list contains data.tables, we don't want to edit the data directly (by ref)
+        if(any(sapply(data,is.data.table))){
+            data <- copy(data)
+        }
         names.data <- names(data)
         if(is.null(names.data)) {
             names.data <- as.character(1:length(data))
@@ -873,7 +890,9 @@ NMsimConf <- NMsimTestConf(path.nonmem=path.nonmem,dir.psn=dir.psn,method.execut
         if(!quiet) message(sprintf("Reading from simulation results on file:\n%s",dt.models[,paste(unique(path.rds),collapse="\n")]))
         
         simres <- try(NMreadSim(dt.models[,path.rds],wait=wait,quiet=quiet,progress=progress,as.fun=as.fun))
-        if(!inherits(simres,"try-error")) {
+        if(inherits(simres,"try-error") || is.null(simres) || nrow(simres)==0) {
+            message("Reading results on file returned error(s). Simulations will be re-run even though `reuse.results=TRUE`.")
+        } else {
             return(returnSimres(simres))
         }
     }
@@ -908,6 +927,15 @@ NMsimConf <- NMsimTestConf(path.nonmem=path.nonmem,dir.psn=dir.psn,method.execut
     }
     
 ### Generate the first version of file.sim.
+    ## if(missing(inits)){
+    ##     inits <- NULL
+    ## }
+
+    ##if(is.null(inits)) inits <- list(method="nmsim",update=TRUE)
+
+    
+    
+    
     ## It would not need to, but beware PSN's update_inits needs to
     ## create a new file - don't try to overwrite an existing one.
     if(method.update.inits=="none"){
@@ -916,7 +944,11 @@ NMsimConf <- NMsimTestConf(path.nonmem=path.nonmem,dir.psn=dir.psn,method.execut
 
 ##### todo all file.xyz arguments must be NULL or of equal length. And this should be done per model
     
-    if(method.update.inits=="nmsim" && any(dt.models[,!file.exists(file.ext)])){
+    if(method.update.inits=="nmsim.deprec" && any(dt.models[,!file.exists(file.ext)])){
+        stop(paste("ext file(s) not found. Did you forget to copy it? Normally, NMsim needs that file to find estimated parameter values. If you do not have an ext file and you are running a simulation that does not need it, please use `method.update.inits=\"none\"`. Was expecting to find ",paste(dt.models[!file.exists(file.ext),file.ext],collapse="\n"),sep=""))
+    }
+
+    if(method.update.inits=="nmsim" && inits$update && any(dt.models[,!file.exists(file.ext)])){
         stop(paste("ext file(s) not found. Did you forget to copy it? Normally, NMsim needs that file to find estimated parameter values. If you do not have an ext file and you are running a simulation that does not need it, please use `method.update.inits=\"none\"`. Was expecting to find ",paste(dt.models[!file.exists(file.ext),file.ext],collapse="\n"),sep=""))
     }
     
@@ -952,13 +984,25 @@ NMsimConf <- NMsimTestConf(path.nonmem=path.nonmem,dir.psn=dir.psn,method.execut
             file.rename(file.path(dirname(file.mod),fn.sim.tmp),path.sim)
         },by=ROWMODEL]
     }
+    
+    if(method.update.inits=="nmsim.deprec"){
+        ## edits the simulation control stream in the
+        ## background. dt.models not affected.
+### because we use newfile, this will be printed to newfile. If not, it would just return a list of control stream lines.
+        dt.models[,NMupdateInits(file.mod=file.mod,newfile=path.sim,file.ext=file.ext),by=.(ROWMODEL)]
+    }
+
     if(method.update.inits=="nmsim"){
         ## edits the simulation control stream in the
         ## background. dt.models not affected.
-
 ### because we use newfile, this will be printed to newfile. If not, it would just return a list of control stream lines.
-        dt.models[,NMupdateInits(file.mod=file.mod,newfile=path.sim,fix=TRUE,file.ext=file.ext),by=.(ROWMODEL)]
 
+        ## dt.models[,NMwriteInits(file.mod=file.mod,newfile=path.sim,file.ext=file.ext,),by=.(ROWMODEL)]
+        dt.models[,{
+            args.inits <- append(list(file.mod=file.mod,newfile=path.sim,file.ext=file.ext),
+                                 inits[setdiff(names(inits),"method")])
+            do.call(NMwriteInits,args.inits)
+        },by=.(ROWMODEL)]
     }
 
     
@@ -977,7 +1021,7 @@ NMsimConf <- NMsimTestConf(path.nonmem=path.nonmem,dir.psn=dir.psn,method.execut
                 message(paste0("Row counter was added in column ",col.row,". Use this to merge output and input data."))
                 section.input <- NMreadSection(file.mod,section="input",keep.name=FALSE)
                 
-                section.input <- paste.begin(x=section.input,paste("$INPUT",col.row),collapse=" ")
+                section.input <- pasteBegin(x=section.input,paste("$INPUT",col.row),collapse=" ")
             } else {
                 section.input <- FALSE
             }
@@ -1103,26 +1147,29 @@ NMsimConf <- NMsimTestConf(path.nonmem=path.nonmem,dir.psn=dir.psn,method.execut
 
 
 ##### Moved to after reuse.results - before NMexec()
-        ## if multiple models have been spawned, and files.needed has been generated, the only allowed method.execute is "nmsim"
-        if(nrow(dt.models.gen)>1 && "files.needed"%in%colnames(dt.models.gen)){
-            if(NMsimConf$method.execute!="nmsim"){
-                stop("Multiple simulation runs spawned, and they need additional files than the simulation input control streams. The only way this is supported is using method.execute=\"nmsim\".")
-            }
+    ## if multiple models have been spawned, and files.needed has been generated, the only allowed method.execute is "nmsim"
+    if(nrow(dt.models.gen)>1 && "files.needed"%in%colnames(dt.models.gen)){
+        if(execute && NMsimConf$method.execute!="nmsim"){
+            stop("Multiple simulation runs spawned, and they need additional files than the simulation input control streams. The only way this is supported is using method.execute=\"nmsim\".")
         }
+    }
 
-        ## if multiple models spawned, direct is not allowed
-        if(nrow(dt.models.gen)>1){
-            if(NMsimConf$method.execute=="direct"){
-                stop("method.execute=\"direct\" cannot be used with simulation methods that spawn multiple simulation runs. Try method.execute=\"nmsim\" or method.execute=\"psn\".")
-            }
-        }
 
-        ## if files.needed, psn execute cannot be used.
-        if("files.needed"%in%colnames(dt.models.gen)){
-            if(NMsimConf$method.execute=="psn"){
-                stop("method.execute=\"psn\" cannot be used with simulation methods that need additional files to run. Try method.execute=\"nmsim\".")
-            }
+    ## if multiple models spawned, direct is not allowed
+    if(nrow(dt.models.gen)>1){
+        if(execute && NMsimConf$method.execute=="direct"){
+            stop("method.execute=\"direct\" cannot be used with simulation methods that spawn multiple simulation runs. Try method.execute=\"nmsim\" or method.execute=\"psn\".")
         }
+    }
+    
+
+    ## if files.needed, psn execute cannot be used.
+    if("files.needed"%in%colnames(dt.models.gen)){
+        if(execute && NMsimConf$method.execute=="psn"){
+            stop("method.execute=\"psn\" cannot be used with simulation methods that need additional files to run. Try method.execute=\"nmsim\".")
+        }
+    }
+    
 
     
     
@@ -1200,28 +1247,33 @@ NMsimConf <- NMsimTestConf(path.nonmem=path.nonmem,dir.psn=dir.psn,method.execut
                 
 ### pasting the seed after SIM(ULATION) and after ONLYSIM(ULATION) if the latter exists
                 section.sim <- sub("(SIM(ULATION)*( +ONLYSIM(ULATION)*)*) *",paste("\\1",seed),section.sim)
-                ## section.sim <- paste.end(section.sim,seed)
-                                        #}
+
                 if(subproblems>0){
                     section.sim <- gsub("SUBPROBLEMS *= *[0-9]*"," ",section.sim)
-                    section.sim <- paste.end(section.sim,sprintf("SUBPROBLEMS=%s",subproblems))
+                    section.sim <- pasteEnd(section.sim,sprintf("SUBPROBLEMS=%s",subproblems))
                 }
                 ## section.sim <- paste(section.sim,text.sim)
-                section.sim <- paste.end(section.sim,text.sim)
+                section.sim <- pasteEnd(section.sim,text.sim)
                 lines.sim <- NMdata:::NMwriteSectionOne(lines=lines.sim,section="simulation",newlines=section.sim,quiet=TRUE,backup=FALSE)
                 writeTextFile(lines.sim,path.sim)
             }
         },by=.(ROWMODEL2)]
     }
-
-
-
+    
+    
+    
 #### Section start: Additional control stream modifications specified by user - modify.model ####
-    if( !is.null(modify.model) ){
-### This requires NMdata >=0.1.0.905
+
+    if(missing(sizes)) sizes <- NULL
+    if(!is.null(sizes)){
         dt.models[,{
-            NMwriteSection(files=path.sim,list.sections=modify.model,quiet=TRUE,backup=FALSE)
-        },by=.(ROWMODEL)]
+            args.sizes <- append(list(file.mod=path.sim,newfile=path.sim,write=TRUE),sizes)
+            do.call(NMupdateSizes,args.sizes)
+        },by=.(ROWMODEL2)]
+    }
+
+    if( !is.null(modify.model) ){
+        modifyModel(modify.model,dt.models)
     }
     
 ### Section end: Additional control stream modifications specified by user - modify.model
@@ -1295,7 +1347,8 @@ NMsimConf <- NMsimTestConf(path.nonmem=path.nonmem,dir.psn=dir.psn,method.execut
                                  ## width = 50,   # Progress bar width. Defaults to getOption("width")
                                  char = "=")
         }
-        
+
+###### execute jobs using NMexec
         dt.models[,lst:={
             simres.n <- NULL
             files.needed.n <- try(strsplit(files.needed,":")[[1]],silent=TRUE)
@@ -1376,6 +1429,9 @@ NMsimConf <- NMsimTestConf(path.nonmem=path.nonmem,dir.psn=dir.psn,method.execut
     ## if(execute && (wait.exec||wait)){
     if(is.NMsimRes(simres) || (execute && (wait.exec||wait))){
         if(!quiet){
+            if(nrow(simres)==0){
+                message("Simulation results are empty. An empty data.frame is returned.")
+            }
             message("\nSimulation results returned. Re-read them without re-simulating using:\n",paste(sprintf("  simres <- NMreadSim(\"%s\")",dt.models[,unique(path.rds)]),collapse="\n"))
         }
         return(returnSimres(simres))
